const express = require('express')
const path = require('path')
const webpack = require('webpack')
const webpackDevMiddleware = require('webpack-dev-middleware')
const webpackHotMiddleware = require('webpack-hot-middleware')
const httpProxy = require('http-proxy-middleware')
const fs = require('fs-extra')
const argv = require('yargs').argv
const bodyParse = require('body-parser')
const chalk = require('chalk')

const app = express()
const config = require('../scripts/webpack/webpack.dev.config.js')
const runOpen = require('./run-open')
const feleWatch = require('./file-watch')
const generateMate = require('./generate-mate')
const compiler = webpack(config)
const generateTsx = require('./generate-form-tsx')
const commands = argv._ || []

const proxyConfig = {
    target: '/',
    changeOrigin: true,
    pathRewrite: {
        '^/api': ''
    },
    context: '/api'
}
const port = process.env.port || '9088'
const middleware = webpackDevMiddleware(compiler, {
    publicPath: config.output.publicPath,
    silent: true,
    stats: {
        modules: false,
        children: false,
        chunks: false,
        chunkModules: false,
        colors: true
    }
})

const resolve = (dir) => path.join(__dirname, '../', dir)

middleware.waitUntilValid((error) => {
    if (commands[0] === 'start') {
        return
    }
    // if (error.hasErrors()){
    //     return
    // }
    console.log(chalk.green('生成配置文件中...\n'))
    feleWatch()
})

app.use(bodyParse.urlencoded())
app.use(bodyParse.json())
app.use(middleware)

app.use(webpackHotMiddleware(compiler))

var devProxy = httpProxy(proxyConfig)

app.use(proxyConfig.context, devProxy)

const middleFs = middleware.fileSystem

app.get('*', (req, res) => {
    middleFs.readFile(path.join(compiler.outputPath, 'index.html'), (err, file) => {
        if (err) {
            res.sendStatus(404)
        } else {
            res.send(file.toString())
        }
    })
})

app.get('*.js', (req, res, next) => {
    req.url = req.url + '.gz'
    res.set('Content-Encoding', 'gzip')
    next()
})

app.post('/modify-mate', async (req, res, next) => {
    const { list, formLayout, formName } = req.body
    const name =
        formName && formName !== 'null' && formName !== 'undefined'
            ? formName
            : 'form-' + parseInt(Math.random() * 1000)
    const mate = await generateMate(list, formLayout, name)
    res.send({ code: 0, data: { mate, name } })
    next()
})

app.post('/import-json', async (req, res, next) => {
    const { data, name } = req.body
    const nameT = name || 'form-' + parseInt(Math.random() * 1000)
    const mate = await generateMate(
        data,
        {
            labelCol: { span: undefined, offset: undefined, push: undefined, pull: undefined },
            wrapperCol: { span: undefined, offset: undefined, push: undefined, pull: undefined },
            layout: 'horizontal'
        },
        nameT
    )
    res.send({ code: 0, data: { mate, nameT } })
    next()
})

app.post('/build-page', async (req, res, next) => {
    try {
        const { list, formLayout, formName } = req.body
        await generateMate(list, formLayout, formName)
        await generateTsx(formName)
        res.send({ code: 0 })
        next()
        runOpen(`http://localhost:${port}/generate`, {
            openPage: '/',
            open: ['google chrome', '--incognito']
        })
    } catch (err) {
        console.error(err)
        res.send({ code: -1, msg: '解析失败' })
    }
})

app.post('/get-meta-details', async (req, res, nest) => {
    const { formName } = req.body
    if (!formName || formName === 'null' || formName === 'undefined') {
        res.send({ code: -1, msg: '名称为空' })
        return
    }
    const fielUrl = path.join(resolve('/cli/meta-file'), `${formName}.json`)
    fs.accessSync(fielUrl)
    const meta = JSON.parse(fs.readFileSync(fielUrl))
    res.send({ code: 0, data: { details: meta } })
})

app.post('/get-meta-list', async (req, res, next) => {
    const names = fs.readdirSync(resolve('/cli/meta-file'))
    res.send({
        code: 0,
        data: {
            metaList: names.map((v) => {
                return v.replace(/.json/, '')
            })
        }
    })
    next()
})

app.post('/add-meta', async (req, res, next) => {
    const { name, doc } = req.body
    const names = fs.readdirSync(resolve('/cli/meta-file'))
    if (names.indexOf(name + '.json') != -1) {
        next()
        return res.send({ code: -1, msg: '有相同名称的表单' })
    }
    fs.writeFileSync(
        path.join(resolve('/cli/meta-file'), `${name}.json`),
        JSON.stringify(
            {
                name,
                doc,
                list: [],
                formLayout: {
                    labelCol: {},
                    wrapperCol: {},
                    layout: 'vertical'
                }
            },
            undefined,
            2
        )
    )
    res.send({
        code: 0,
        data: {
            metaList: [
                ...names.map((v) => {
                    return v.replace(/.json/, '')
                }),
                name
            ]
        },
        msg: '保存成功'
    })
    next()
})

// Serve the files on port 3000.
app.listen(port, function(err) {
    if (err) {
        throw err
    }
    runOpen(`http://localhost:${port}`, {
        openPage: '/',
        open: ['google chrome', '--incognito']
    })
})
